<?php
/**
 * @file
 * Include functions for the guild module.
 */

/**
 * Callback used to sort an array of characters by rank.
 *
 * The lowest the highest.
 */
function _wow_character_rank_cmp($a, $b) {
  if ($a->rank == $b->rank) {
    return 0;
  }
  return ($a->rank < $b->rank) ? -1 : 1;
}

/**
 * Save guild members; Process ranks and user roles.
 *
 * @param object $guild
 *   Guild object as fetched by the service. Must contains 'members' array.
 */
function wow_guild_members_save($guild) {
  // Load every known members of the guild from the local database.
  $characters = db_select('wow_characters', 'c')
    ->fields('c')
    ->condition('c.gid', $guild->gid)
    ->execute()
    ->fetchAllAssoc('name');

  // Load guild ranks if needed.
  $use_ranks = module_exists('wow_guild_ranks') && wow_guild_is_default($guild);
  $mains = array();

  // Process the member list from the service.
  usort($guild->members, '_wow_character_rank_cmp');
  foreach ($guild->members as $member) {
    $character = $member->character;
    $character->region = $guild->region;

    $op = FALSE;
    $key = $character->name;
    if (array_key_exists($key, $characters)) {
      // The member already exists in the local database. For efficiency
      // manually save the original character before applying any changes.
      $character->original = $characters[$key];

      if ($member->rank != $characters[$key]->rank) {
        // The member rank changed since last update.
        $character->rank = $member->rank;
        $op = 'rank_update';
      }

      $character->cid = $characters[$key]->cid;

      // Remove character from the array if they are found. Remaining array
      // contains the list of players who are not in the guild anymore.
      unset($characters[$key]);
    }
    else {
      // The member is not in the list of character having this guild as gid.
      // Try to load it anyway by name and realm.
      $original = wow_character_load_by_name($character->realm, $character->name);

      if ($original) {
        // This character already exists in the db, but yet not linked with
        // the guild. Set the cid so we know this is an update.
        $character->original = $original;
        $character->cid = $character->original->cid;
      }

      $character->gid = $guild->gid;
      $character->rank = $member->rank;

      $op = 'member_joined';
    }

    // Character is associated with a user and active.
    if ($use_ranks && !empty($character->original->uid) && !empty($character->original->status)) {
      // Characters are sorted by ranks: the first time a user character is
      // encountered, he is the main. The following characters for the same
      // user will have a rank lower or equal to the first one; they are
      // alternatives characters.
      $main = empty($mains[$character->original->uid]);
      $mains[$character->original->uid] = TRUE;

      if ($main != $character->original->is_main) {
        // The main status changed since last update.
        $character->is_main = $main;
      }
    }

    // Achievement points are buggy when retrieved from a guild update.
    // TODO: Re-set them when blizzard fix the bug.
    unset($character->achievementPoints);

    if ($op) {
      // Let modules act on characters who are in the guild before it is saved to the database.
      module_invoke_all("wow_guild_$op", $guild, $character);
    }

    // Save new or existing characters into database.
    wow_character_save($character);
  }

  // Now process the characters from the remaining array. This remaining array
  // is composed of user which was on the database but not found in the response
  // from service. This mean they are not in the guild anymore.
  foreach ($characters as $character) {
    $character->original = clone $character;

    if ($use_ranks && !empty($character->original->uid) && !empty($character->original->status) && $character->is_main) {
      // When a main character leave the guild, the main status is also revoked.
      $character->is_main = FALSE;
    }

    // Set the guild and rank as anonymous values.
    $character->gid = 0;
    $character->rank = 0;

    // Let modules act on characters who left the guild before it is saved to the database.
    module_invoke_all('wow_guild_member_left', $guild, $character);

    // Save characters who left the guild.
    wow_character_save($character);
  }

  if (!empty($mains)) {
    // Check for each user in the guild if they have a main character which not
    // belongs to the guild. In this case the main status must be revoked.
    $cids = db_select('wow_characters', 'c')
      ->fields('c', array('cid'))
      ->condition('is_main', TRUE)
      ->condition('gid', $guild->gid, '<>')
      ->condition('uid', array_keys($mains))
      ->execute()
      ->fetchCol();

    foreach (wow_character_load_multiple($cids) as $character) {
      $character->original = clone $character;
      $character->is_main = FALSE;
      wow_character_save($character);
    }
  }
}
